package calc;

enum State {
	S0, S1, S2, S3;
}

enum Operator {
	NoOp, Plus, Minus, Mul, Div;
}

enum Input {
	KeyNum, KeyOperator, KeyClr, KeyEq;
}

public class CalcMain {
	State state;
	int mem, curr;
	Operator op;
	
	public CalcMain() {
		mem = curr = 0;
		op = Operator.NoOp;
		state = State.S0;
	}
	
	public void doState(Input key, int v, Operator o) {
		switch(state) {
		case S0:
			switch(key) {
			case KeyNum:
				state = State.S1;
				curr = v;
				break;
			case KeyOperator:
				state = State.S2;
				op = o;
				break;
			case KeyClr:
				state = State.S0;
				mem = curr = 0;
				break;
			case KeyEq:
				state = State.S0;
				break;
			}
			break;
		case S1:
			switch(key) {
			case KeyNum:
				state = State.S1;
				curr = curr * 10 + v;
				break;
			case KeyOperator:
				state = State.S2;
				op = o;
				break;
			case KeyClr:
				state = State.S0;
				mem = curr = 0;
			case KeyEq:
				state = State.S0;
				break;
			}
			break;
		case S2:
			switch(key) {
			case KeyNum:
				state = State.S3;
				mem = curr;
				curr = v;
				break;
			case KeyOperator:
				state = State.S2;
				op = o;
				break;
			case KeyClr:
				state = State.S0;
				mem = curr = 0;
				break;
			case KeyEq:
				state = State.S0;
				break;
			}
			break;
		case S3:
			switch(key) {
			case KeyNum:
				state = State.S3;
				curr = curr * 10 + v;
				break;
			case KeyOperator:
				state = State.S2;
				curr = doCalc();
				op = o;
				break;
			case KeyClr:
				state = State.S0;
				mem = curr = 0;
				break;
			case KeyEq:
				state = State.S0;
				curr = doCalc();
				break;
			}
		}
	}
	
	private int doCalc() {
		int r = 0;
		switch(op) {
		case Plus:
			r = mem + curr;
			break;
		case Minus:
			r = mem - curr;
			break;
		case Mul:
			r = mem * curr;
			break;
		case Div:
			r = mem / curr;
			break;
		}
		return r;
	}
	
	public int getCurrent() {
		return curr;
	}
}
